//                                                                                                                            
// ********************************************************************                                                       
// *                                                                  *                                                       
// * Physical Target and Detector Definitions                         *                                                       
// *                                                                  *                                                       
// ********************************************************************                                                       
//                                                                                                                            

#include "myDetectorConstruction.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4MultiFunctionalDetector.hh"
#include "G4VPrimitiveScorer.hh"
#include "G4PSSphereSurfaceCurrent.hh"
#include "G4SDParticleFilter.hh"
#include "G4SDManager.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4ios.hh"
#include "G4NistManager.hh"
#include <stdio.h>
//#include "triPolRingSD.hh"
#include "G4UniformMagField.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
#include "G4SubtractionSolid.hh"

#include "G4SystemOfUnits.hh"
// ********************************************************************                                                       

myDetectorConstruction::myDetectorConstruction() { }

// ********************************************************************                                                       

myDetectorConstruction::~myDetectorConstruction() { }

// ********************************************************************                 
G4VPhysicalVolume* myDetectorConstruction::Construct()
{

  //--------- Material definition ---------                                                                                     

  G4double density;

  G4NistManager* man = G4NistManager::Instance();


  G4Material* Beryllium = man->FindOrBuildMaterial("G4_Be");
  //G4Material* Blood = man->FindOrBuildMaterial("G4_BLOOD_ICRP");


  G4double atomicNumber=1.;
  G4double massOfMole=1.008*g/mole;
  density=1.e-25*g/cm3;
  G4double temperature=2.73*kelvin;
  
  G4double pressure=3.e-18*pascal; //SPACE                                                                                  
  //G4double pressure=1.e-3*pascal;
  G4Material *Vacuum =  new G4Material("Vacuum",atomicNumber,massOfMole,density,kStateGas,temperature,pressure);

  //G4Material* Blood = man->FindOrBuildMaterial("G4_BLOOD_ICRP");  
  //G4Material* Carbon = man->FindOrBuildMaterial("G4_C");
  G4Material* CsI = man->FindOrBuildMaterial("G4_CESIUM_IODIDE");

  // Print all the materials defined.                                                                             
            
  G4cout << G4endl << "The materials defined are : " << G4endl << G4endl;
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  //--------- Definitions of Solids, Logical Volumes, Physical Volumes ---------                                                

  //------------------------------                                                                                            
  // World                                                                                                                    
  //------------------------------                                                                                            

  G4ThreeVector positionWorld  = G4ThreeVector(0,0,0);
  solidWorld = new G4Box("world",50*cm,50*cm,75*cm);
  logicWorld = new G4LogicalVolume(solidWorld,Vacuum,"World",0,0,0);
  physiWorld = new G4PVPlacement(   0, // rotate matrix
						       positionWorld,     // at (x,y,z)
						       logicWorld,        // its logical volume
						       "World",           // its name
						       0,                 // its mother  volume
						       false,             // no boolean operations
						       0);                // copy number
 
  //G4ThreeVector positionFoil  = G4ThreeVector(0*cm,0.0*cm,0.0*cm);
  //G4Box *solidFoil = new G4Box("Foil",10*cm,10*cm,thickness);
  //G4LogicalVolume *logicFoil = new G4LogicalVolume(solidFoil,Beryllium,"Foil",0,0,0);
  //G4VPhysicalVolume *physiFoil = new G4PVPlacement(   0, // rotate matrix
  //							 positionFoil,     // at (x,y,z)
  //							 logicFoil,        // its logical volume
  //							 "Foil",           // its name
  //							 logicWorld,        // its mother  volume
  //							 false,             // no boolean operations
  //							 0);                // copy number
  //make array
  int n = 100;
  int nRows = 10; //make sqrt(n)

  //G4ThreeVector positionTube [100];
  G4ThreeVector positionTube;
  //G4VPhysicalVolume *physiTube [100];


  char tubeName[80];
  //for(int i=0; i<n; ++i) {
  // int col = i%nRows;
  //int row = i/nRows;

  for(int iRow=0; iRow<nRows; ++iRow) {
    for(int iCol=0; iCol<nRows; ++iCol) {
    
      //positionTube[i] = G4ThreeVector(col*cm,row*cm,15*cm);
      //*physiTube[i] = new G4PVPlacement(0,positionTube,logicTube,"Tube",logicWorld,false,0);
      G4double deltaX = 2*cm;
      G4double deltaY = 2*cm;
      //G4double xpos = iRow*deltaX - nRows*deltaX/2 + deltaX/2 + 30*cm;
      //G4double ypos = iCol*deltaY - nRows*deltaY/2 + deltaY/2 + 30*cm;

      G4double xpos = iRow*deltaX - nRows*deltaX/2 + deltaX/2;
      G4double ypos = iCol*deltaY - nRows*deltaY/2 + deltaY/2;

      positionTube = G4ThreeVector(xpos,ypos,25*cm);
      //sprintf(tubeName,"tube_%d_%d",row,col);
      sprintf(tubeName,"%d",(iRow+1)*100+(iCol+1));
      
      //G4cout<<tubeName<<" c, r = "<<col<<" , "<<row<<G4endl;
      
      G4VisAttributes* tubeVisAtt= new G4VisAttributes(G4Colour(0.0,0.0,0.0));
      tubeVisAtt->SetForceSolid(false);
      
      G4Box *solidTube = new G4Box (tubeName,1.0*cm,1.0*cm,10*cm);
      G4LogicalVolume *logicTube = new G4LogicalVolume(solidTube,CsI,tubeName,0,0,0);   
      G4VPhysicalVolume *physiTube = new G4PVPlacement(   0, // rotate matrix
							  positionTube,     // at (x,y,z)
							  logicTube,        // its logical volume
							  tubeName,           // its name
							  logicWorld,        // its mother  volume
							  false,             // no boolean operations
							  0);                // copy number
      
      //if (i == 50) 
      logicTube->SetVisAttributes(tubeVisAtt);
    }
  }


  //QQQQ
  G4ThreeVector positionFVol  = G4ThreeVector(0*cm,0*cm,10*cm);
  G4Box *solidFVol = new G4Box ("solidFVol",10.0*cm,10.0*cm,5*cm);
  G4LogicalVolume *logicFVol = new G4LogicalVolume(solidFVol,Vacuum,"logicFVol",0,0,0);
  G4VPhysicalVolume *physiFVol = new G4PVPlacement(   0, // rotate matrix
						      positionFVol,     // at (x,y,z)
						      logicFVol,        // its logical volume
						      "What",           // its name
						      logicWorld,        // its mother  volume
						      false,             // no boolean operations
						      0);                // copy number
  
  G4VisAttributes* fVolVisAtt= new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  fVolVisAtt->SetForceSolid(false);
  logicFVol->SetVisAttributes(fVolVisAtt);

  G4UniformMagField* magField1;
  G4double magVal1 = 2*tesla;
  magField1 = new G4UniformMagField(G4ThreeVector(magVal1,0.0,0.0));
  G4FieldManager *localFieldMgr1 = new G4FieldManager(magField1);
  localFieldMgr1->SetDetectorField(magField1);
  localFieldMgr1->CreateChordFinder(magField1);
  logicFVol->SetFieldManager(localFieldMgr1,true);

  //------------------------------                                                                                            
  // Converter                                                                                                                
  //------------------------------                             

  /* G4double innerRadiusOfConverter = 0.*cm;
  G4double outerRadiusOfConverter = 0.5*cm;
  G4double heightOfConverter;
  heightOfConverter = 1.0*cm/2.0;
  G4double startAngleConverter = 0.*deg;
  G4double spanningAngleConverter = 360.*deg;
  G4ThreeVector positionConverter  = G4ThreeVector(0.0*cm,0.0*cm,0*cm);

  G4Tubs* solidConverter = new G4Tubs("converter",
				      innerRadiusOfConverter,
				      outerRadiusOfConverter,
				      heightOfConverter,
				      startAngleConverter,
				      spanningAngleConverter);
  G4LogicalVolume* logicConverter = new G4LogicalVolume(solidConverter,Carbon,"converter");
  G4VPhysicalVolume* physiConverter = new G4PVPlacement(0,               // rotation matrix
							positionConverter,  // at (x,y,z)
							logicConverter,     // its logical volume
							"converter",        // its name
							logicWorld,      // its mother  volume
							false,           // no boolean operations
							0);              // copy number
                                                             
  */


  //--------- Visualization attributes -------------------------------   
  G4VisAttributes* WorldVisAtt= new G4VisAttributes(G4Colour(0.0,0.0,0.0));
  logicWorld->SetVisAttributes(WorldVisAtt);
  // logicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4VisAttributes* ConverterVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,0.0));
  //ConverterVisAtt->SetForceSolid(true);
  //logicConverter->SetVisAttributes(ConverterVisAtt);          

  G4VisAttributes* redSolidVA = new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  redSolidVA->SetForceSolid(true);

  G4VisAttributes* greenSolidVA = new G4VisAttributes(G4Colour(0.0,1.0,0.0));
  greenSolidVA->SetForceSolid(true);

  G4VisAttributes* blueSolidVA = new G4VisAttributes(G4Colour(0.0,0.0,1.0));
  blueSolidVA->SetForceSolid(true);

  G4VisAttributes* whatSolidVA = new G4VisAttributes(G4Colour(0.5,0.0,1.0));
  whatSolidVA->SetForceSolid(true);

  //QQQQ
  //logicFVol->SetVisAttributes(redSolidVA);

  //logicFoil->SetVisAttributes(redSolidVA);

  //logicTube->SetVisAttributes(greenSolidVA);

  //logicConverter->SetVisAttributes(blueSolidVA);          
  //logicBloodbox->SetVisAttributes(greenSolidVA);          
  //logicBloodbox->SetVisAttributes(whatSolidVA);          
  //logicBloodbox->SetVisAttributes(G4VisAttributes::Invisible);          


  return physiWorld;
}


void myDetectorConstruction::SetRowColumnID(G4String volName)
{
  crystalRowID = -1;
  crystalColID = -1;

  int rcID = atoi(volName);
  int colNumTmp  = rcID % 100;
  int rowNumTmp = (rcID - colNumTmp)/100;
  if (colNumTmp >= 1 && colNumTmp <= 10) crystalColID = colNumTmp -1;
  if (rowNumTmp >= 1 && rowNumTmp <= 10) crystalRowID   = rowNumTmp -1;
}








