//
// ********************************************************************
// *                                                                  *
// *       Primary Generator (source of beam)                         *
// *                                                                  *
// ********************************************************************
//

#include "myPrimaryGeneratorAction.hh"
#include "myDetectorConstruction.hh"
#include "Randomize.hh"
#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "globals.hh"
#include "TTree.h"
#include "outTree.h"
#include "TH1.h"
#include "TH2.h"
#include "TLorentzVector.h"
#include "G4SystemOfUnits.hh"





// ********************************************************************

myPrimaryGeneratorAction::myPrimaryGeneratorAction()
{

  G4String particle_name; 
  particle_name = "proton"; 
  G4int number_particle = 1;
  particleGun = new G4ParticleGun(number_particle);
  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* particle = particleTable->FindParticle(particle_name);
  particleGun->SetParticleDefinition(particle);


}

// ********************************************************************

myPrimaryGeneratorAction::~myPrimaryGeneratorAction()
{
  delete particleGun;
}

// ********************************************************************


void myPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{  

  G4double Ekin;

  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4ParticleDefinition* particleE = particleTable->FindParticle("e-");
  G4ParticleDefinition* particlePro = particleTable->FindParticle("proton");
  
  posX = 0.0*cm;
  posY = 0.0*cm;
  posZ = -10.0*cm;
  posX = G4UniformRand()*20*cm - 10*cm;
  posY = G4UniformRand()*20*cm - 10*cm;

  //posX = 1.0*cm;
  //posY = 1.0*cm;

  G4double mElec = 0.511*MeV;
  G4double mPro = 0.938*GeV;
  //Ekin = sqrt(beamMomentum*beamMomentum + mPro*mPro) - mPro;//FOR PROTON
  Ekin = sqrt(beamMomentum*beamMomentum + mElec*mElec) - mElec;//FOR ELECTRON

  beamEnergy = Ekin;

  //SHOOT electron
  particleGun->SetParticlePosition(G4ThreeVector(posX,posY,posZ));//PSOTION OF ORIGINATION
  particleGun->SetParticleDefinition(particleE);//SET THE PARTICLE AS ELECTRON
  particleGun->SetParticleEnergy(Ekin);//SET KINETIC ENERGY
  particleGun->SetParticleMomentumDirection(G4ThreeVector(0,0,1));//MOMENTUM DIRECTION
  particleGun->GeneratePrimaryVertex(anEvent);//SHOOT THE GUN




}

// ********************************************************************

